package client;

import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * A ClientClusterSet object is a set of cluster which was found
 *	 by a previous search execution.
 */	
public class ClientClusterSet implements Serializable {	
	private ClientSearchParameterSet csp;
	private ArrayList list;
	private HashMap mapDetails;
	private String directory;
	
	/**
	 * Construct a new ClientClusterSet object.
	 */
	public ClientClusterSet() {
		this.csp = new ClientSearchParameterSet();
		this.list = new ArrayList();
		this.mapDetails = new HashMap();
		this.directory = "";
	}

	/**
	 * Construct a new ClientClusterSet object.
	 * @param csp the set of search parameters associated with the cluster set
	 */
	public ClientClusterSet(ClientSearchParameterSet csp) {
		this.csp = csp;
		this.list = new ArrayList();
		this.mapDetails = new HashMap();
		this.directory = "";
	}
	
	/**
	 * Get the default directory for this cluster set. 
	 * @return the directory path
	 */	
	public String getDirectory() {
		return this.directory;
	}
	
	/**
	 * Set a new default directory for this cluster set. 
	 * @param directory the new directory path to set
	 */	
	public void setDirectory(String directory) {
		this.directory = directory;
	}
	
	/**
	 * Get the specie name which was search to find this cluster set.
	 * @return the specie name in a String object
	 */	
	public String getSpecie() {
		return csp.getSpecieSearch();
	}
	
	/**
	 * Get the number of cluster in the cluster set.
	 * @return the number of cluster prresent in the set
	 */
	public int getNbCluster(){
		return list.size();
	}
	
	/**
	 * Get the cluster list.
	 * @return the ArrayList containing all the clusters.
	 */	
	public ArrayList getClusterList() {
		return this.list;
	}
	
	/**
	 * Set the cluster list.
	 * @param list the ArrayList containing all the clusters to set.
	 */
	public void setClusterList(ArrayList list){
		this.list = list;
	}
	
	/**
	 * Get the cluster at speciefied position in the set.
	 * @param n the position of the cluster in the set
	 * @return the cluster at the specified position 
	 */	
	public ClientCluster getCluster(int n) {
		return ((ClientCluster)list.get(n));
	}
	
	/**
	 * Get the EST matching details vector for a cluster of the set.
	 * @param c the cluster to retrieve matching details
	 * @return a vector containing details of matching for a cluster
	 */
	public int[] getVectorDetail(ClientCluster c) {
		return (int[])this.mapDetails.get(""+c.getClusterCode());	
	}
	
	/**
	 * Set the complete vector HashMap for all the clusters in the set 
	 *	This map contains all the details about cluster's appartenance.
	 * @param details the detail's HashMap to set.
	 */
	public void setVectorMap(HashMap details) {
		this.mapDetails = details;
	}
	
	/**
	 * Get the complete vector HashMap for all the clusters in the set 
	 *	This map contains all the details about cluster's appartenance.
	 * @return a HashMap containing all the cluster's details.
	 */
	public HashMap getVectorMap() {
		return this.mapDetails;
	}
	
	/**
	 * Get the set of search parameters 
	 *	which was used to determine this cluster set.
	 * @return a ClientSearchParameterSet object containing search parameters
	 */
	public ClientSearchParameterSet getSearchParameterSet() {
		return this.csp;
	}
	
	/**
	 * Get the EST matching details in a String for a cluster of the set.
	 * @param c the cluster to retrieve matching details
	 * @return a String representation of the matching details
	 */
	public String getNbESTDetails(ClientCluster c) {
		if (!mapDetails.isEmpty()) {
			int[] vector;
			if (this.mapDetails.get(""+c.getClusterCode()) == null)
				return "";
			vector = (int[])this.mapDetails.get(""+c.getClusterCode());
			String detail = "";
			for (int i = 1; i < vector.length; ++i) {
				detail += vector[i] + " - ";
			}
			return detail.substring(0, detail.length()-2);
		} else return "";
	}
	
	/**
	 * Get the total number of EST matching for a cluster of the set.
	 * @param c the cluster to retrieve matching details
	 * @return the totzl number of EST matching
	 */	
	public int getTotalNbESTMatching(ClientCluster c) {		
		int[] vector = (int[])mapDetails.get(""+c.getClusterCode());	
		return vector[0];		
	}
	
	/**
	 * Add a new cluster in the list by using "an sorted insertion".
	 * @param c the cluster to add in the set.
	 */	
	public void addCluster(ClientCluster c) {
		
		this.list.add(c);
		
	}
	
	/**
	 * Add a new cluster in the list at a specified position.
	 * @param c the cluster to add in the set
	 * @param position the position to add cluster
	 */	
	public void addClusterAt(ClientCluster c, int position) {
		this.list.add(position, c);		
	}
	
	/**
	 * Delete a cluster from the set at a specified position.
	 * @param n the position of the cluster to delete
	 */	
	public void delCluster(int n) {
		if (!(n < 0 || n >= list.size()))	
			list.remove(n);
	}

	/**
	 * Get in a string the representation of the content of the cluster set.
	 * @return the representation of the content of the cluster set
	 */
	public String toString() {
		if(list.isEmpty()) return "*Empty*";
		else {
		  String ret = "ClusterSet -> "+csp.getSpecieSearch()+" ->";
		  for (int i = 0; i < list.size(); ++i)
		    ret += list.get(i).toString();
		  return ret;
		}
	}

	/**
	 * Serialize in a file a ClientParameters object.
	 * @param cs the ClientClusterSet object to serialize
	 * @param dir the output location directory 
	 */
	public static void saveClusterSet(ClientClusterSet cs, String dir) {
	  try {
	   	cs.setDirectory(dir);
	    FileOutputStream fos = new FileOutputStream(dir);
	    ObjectOutputStream oos = new ObjectOutputStream(fos);
	    oos.writeObject(cs);
	    oos.close();    
	  }
	  catch(IOException e) { }
	}

	/**
	 * Retrieve a serialized ClientClusterSet object from a file.
	 * @param dir the file containing the object
	 * @return the ClientClusterSet object extracted
	 */
	public static ClientClusterSet getClusterSet(String dir) {
	  try {
	    FileInputStream fis = new FileInputStream(dir);
	    ObjectInputStream ois = new ObjectInputStream(fis);
	    ClientClusterSet ret = (ClientClusterSet)ois.readObject();
	    ois.close();
	    return ret;
	  }
	  catch(IOException e) { 
	  	return null;
	  }
	  catch(ClassNotFoundException e) {	
	  	return null;
	  }
	  catch(NullPointerException e) {
	  	return null;
	  }
	}
}
	  