package client;

import java.sql.*;
import java.lang.Runtime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;
import javax.swing.JOptionPane;
import javax.swing.ImageIcon;

/**
 * This class manage the execution of DML & SEL requests on the database.
 */
public class ClientRequest {
	private boolean PRINT_DEBUG_INFO;	
	private JOptionPane connectionClosed;
	protected Connection con;	
	protected Statement stmt;
	protected ResultSet results;
	
	/**
   * Construct a new ClientRequest object. 
   * @param con the established connection
   * @param cp the current parameters for the program
   */
	public ClientRequest(Connection con, ClientParameters cp) {
		this.PRINT_DEBUG_INFO = cp.PRINT_DEBUG_INFO;
	  this.con = con;	
	}

	/**
	 * Execute a selection request on the database.
	 * @param str the SQL request to execute
	 */
	public void doRequestSEL(String str) {
    ExecSEL(str);
  }
	
	/**
	 * Execute a selection request on the database.
	 * @param sqlText the SQL Selection request to execute
	 */	
	private void ExecSEL(String sqlText) {			
		try {
			stmt = con.createStatement();
			results = stmt.executeQuery(sqlText);		  		    
		}
		catch(SQLException e) {
			PrintSQLText(sqlText);
			PrintSQLError(e);
		}
	}
	
	/**
	 * Run a data manipulating request on the database.
	 * @param str the SQL request to execute
	 */
	public void doRequestDML(String str) {
    ExecDML(str);
  }
  
  /**
	 * Execute a data manipulating request on the database.
	 * @param sqlText the SQL Data Manipulating request to execute
	 */	
	private void ExecDML(String sqlText) {		
	  int resCode = -1;		
		try {
		 	stmt = con.createStatement();
		  resCode = stmt.executeUpdate(sqlText);		  
		}
		catch(SQLException e) {
			PrintSQLText(resCode, sqlText);
			PrintSQLError(e);
		}
	}
	
	/**
	 * Lock a table of the database to prevent client access 
	 *	when the server is updated.
	 * @param tableName the name of the table to lock
	 */
	public void lockTable(String tableName) {
		doRequestDML("LOCK TABLES `"+tableName+"` WRITE;");
	}

	/**
	 * Lock tables of the database to prevent client access 
	 *	when the server is updated.
	 * @param tableNames the name list of the tables to lock and the lock mode
	 *	(READ or WRITE)
	 */
	public void lockTables(String[][] tableNames) {
		String lock = "";
		for (int i = 0; i < tableNames.length; ++i) {
			lock += "`"+tableNames[i][0]+"` "+tableNames[i][1];
			if (i == tableNames.length-1) lock += ";";
			else lock += ",";
		} 
		doRequestDML("LOCK TABLES "+lock);
	}
	
	/**
	 * Unlock locked tables of the database.
	 */
	public void unlockTables() {
		doRequestDML("UNLOCK TABLES;");
	}

	/**
	 * Print SQL Text when an exception occurs.
	 * @param sqlText the SQL text to print
	 */
	public void PrintSQLText(String sqlText) {
		if (PRINT_DEBUG_INFO)
	  	System.out.println("ERROR SEL: "+sqlText);
	}	
	
	/**
	 * Print SQL Text when an exception occurs.
	 * @param resultCode the SQL result code for the request
	 * @param sqlText the SQL text to print
	 */
	public void PrintSQLText(int resultCode, String sqlText) {
		if (PRINT_DEBUG_INFO)
	  	System.out.println("ERROR DML ("+resultCode+"): "+sqlText);
	}

	/**
	 * Print a catched SQL Exception.
	 * @param ex the SQL exception to print
	 */
	public void PrintSQLError(SQLException ex) {		
		if (PRINT_DEBUG_INFO) {
	  	System.out.println("** ERROR SQLException :");
	  	while (ex != null) { 
	   		System.out.println("* State  : " + ex.getSQLState());
	    	System.out.println("* Message: " + ex.getMessage());
	    	System.out.println("* Error  : " + ex.getErrorCode());
	    	testConnectionClosed(ex.getErrorCode());	    	
	    	ex = ex.getNextException();
	    	System.out.println();
	  	}
	  }
	}
	
	/**
	 * Test if the database connection is lost & tell the user. 
	 * @param ex the SQL exception to print
	 */
	private void testConnectionClosed(int errorCode) {
		if (errorCode == 0 && connectionClosed == null) {
  		String msg = "Connection to 'bio_db' lost !\n"+
  				"The server may be down !\n\n"+
  				"Please save your search and exit bioECS...";
    	connectionClosed = new JOptionPane();
    	connectionClosed.showMessageDialog(null, msg, 
    		"Connection lost!", JOptionPane.WARNING_MESSAGE, 
    	new ImageIcon("./ico/bioecs.png"));     
  	}
	}
}

