package client;

import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
 * The ClientRequestManager's class is used to execute common request on the database.
 *	The connection to data base is done before executing any other requests.
 * 	Request are from 2 types : 
 *	- Selection requests
 *	- Data manipulating requests
 */
public class ClientRequestManager extends ClientRequest {
	
	/**
   * Default constructor for ClientRequestManager. 
   * Construct a new ClientRequestManager object which can execute request
   * @param con the established connection
   * @param cp the current parameters for the program
   */
	public ClientRequestManager(Connection con, ClientParameters cp) {
		super(con, cp);
	}

	/**
	 * Close connection to database.
	 */
	public void closeConnection() {
  	try {
  		this.stmt.close();
	    this.con.close();
	 	}
	  catch (SQLException e) {
	  	PrintSQLError(e);
	  }
	}

	/** 
	 * Retrieve tissue's code and name for a specie.
	 * @param specieCode the selected specie 
	 * @return a HashMap containing (code, name) values
	 */
	public HashMap get_code_converter_tissue(String specieCode) {
		try {
			stmt = con.createStatement();
		  results = stmt.executeQuery(
		  	"SELECT `tissue_cc`, `code_tissue_cc` "+
		  	" FROM `"+specieCode+"_cc_tissue`;");			    
		  return getResultsInHashMap(results, false);
		}
		catch (SQLException e) { 
			PrintSQLError(e);
			return null;
		}
	}	
	
	/** 
	 * Retrieve tissue's code and name for a specie matching with the pattern.
	 * @param specieCode the selected specie 
	 * @param pattern the pattern to match with
	 * @return a HashMap containing (code, name) values
	 */
	public HashMap get_code_converter_tissue(String specieCode, String pattern) {
		try {
			stmt = con.createStatement();
		  results = stmt.executeQuery(
		  	"SELECT `tissue_cc`, `code_tissue_cc` "+
		  	" FROM `"+specieCode+"_cc_tissue` "+
		  	" WHERE `tissue_cc` LIKE '%"+pattern+"%';");			    
		  return getResultsInHashMap(results, false);
		}
		catch (SQLException e) { 
			PrintSQLError(e);
			return null;
		}
	}

	/** 
	 * Retrieve development stage's code and name for a specie.
	 * @param specieCode the selected specie 
	 * @return a HashMap containing (code, name) values
	 */
	public HashMap get_code_converter_dev_stage(String specieCode) {
		try {
			stmt = con.createStatement();
		  results = stmt.executeQuery(
		  	"SELECT `dev_stage_cc`, `code_dev_stage_cc` "+
		  	" FROM `"+specieCode+"_cc_dev_stage`;");			    
		  return getResultsInHashMap(results, false);
		}
		catch (SQLException e) { 
			PrintSQLError(e);
			return null;
		}
	}
	
	/** 
	 * Retrieve development stage's code and name for a specie matching with the pattern.
	 * @param specieCode the selected specie 
	 * @param pattern the pattern to match with
	 * @return a HashMap containing (code, name) values
	 */
	public HashMap get_code_converter_dev_stage(String specieCode, String pattern) {
		try {
			stmt = con.createStatement();
		  results = stmt.executeQuery(
		  	"SELECT `dev_stage_cc`, `code_dev_stage_cc` "+
		  	" FROM `"+specieCode+"_cc_dev_stage` "+
		  	" WHERE `dev_stage_cc` LIKE '%"+pattern+"%';");			    
		  return getResultsInHashMap(results, false);
		}
		catch (SQLException e) { 
			PrintSQLError(e);
			return null;
		}
	}	
	
	/** 
	 * Retrieve title's code and name for a specie.
	 * @param specieCode the selected specie 
	 * @return a HashMap containing (code, name) values
	 */
	public HashMap get_code_converter_title(String specieCode) {
		try {
			stmt = con.createStatement();
		  results = stmt.executeQuery(
		  	"SELECT `title_cc`, `code_title_cc` "+
		  	" FROM `"+specieCode+"_cc_title`;");			    
		  return getResultsInHashMap(results, false);
		}
		catch (SQLException e) { 
			PrintSQLError(e);
			return null;
		}
	}
	
	/** 
	 * Retrieve the wage of each tissue for a specie.
	 * @param specieCode the selected specie 
	 * @return a HashMap containing (code, wage) values
	 */
	public HashMap get_wage_tissue(String specieCode) {
		try {
			stmt = con.createStatement();
		  results = stmt.executeQuery(
		  	"SELECT `code_tissue_cc`, `tissue_wage` "+
		  	" FROM `"+specieCode+"_cc_tissue`;");			    
		  return getResultsInHashMap(results, true);
		}
		catch (SQLException e) { 
			PrintSQLError(e);
			return null;
		}
	}
	
	/** 
	 * Retrieve the wage of each development stage for a specie.
	 * @param specieCode the selected specie 
	 * @return a HashMap containing (code, wage) values
	 */
	public HashMap get_wage_dev_stage(String specieCode) {
		try {
			stmt = con.createStatement();
		  results = stmt.executeQuery(
		  	"SELECT `code_dev_stage_cc`, `dev_stage_wage` "+
		  	" FROM `"+specieCode+"_cc_dev_stage`;");			    
		  return getResultsInHashMap(results, true);
		}
		catch (SQLException e) { 
			PrintSQLError(e);
			return null;
		}
	}
	
	/**
	 * Get the content of the installed species table.
	 * @return an 2D table containing information about species.
	 */
	public String[][] get_INSTALLED_SPECIES() {
		doRequestSEL(
			"SELECT * FROM `available_species` "+
			" WHERE `ins_ncbi` > -1 AND `ins_tigr` > -1;");
		return toStringTable(getResultsInArrayList(results));
	}
	
	/**
	 * Get the content of the installed species information table for a specie.
	 * @param the NCBI specie code to retrieve instzalled information
	 * @return a String containing information about species.
	 */
	public String get_INSTALLED_SPECIE_INFO(String specieCode) {
		String text = new String("");
		doRequestSEL(
			"SELECT `info` FROM `available_species_info` "+
			" WHERE `code_specie` = '"+specieCode+"';");
		text += getResultsInStringBuffer(results).toString();
		return text;
	}
	
	/**
	 * Add in the data base the ip of the client connected.
	 * @param ip the client ip.
	 */
	public void add_Client_IP(String ip) {
		doRequestDML(
			"INSERT INTO `client_ip` (`ip`) "+
			" VALUES ('"+ip.replace('.', '_')+"');");
	}
	
	/**
	 * Delete client (associated with the ip) temporary created tables.
	 * @param ip the client ip.
	 */
	public void deleteClientTables(String ip) {	    	     	
    doRequestDML(
    	"DROP TABLE IF EXISTS `client_tmp_"+ip+"`;");
    for (int i = 0; i < 30; ++i) {
    	doRequestDML(
    		"DROP TABLE IF EXISTS `client_ncresult"+i+"_"+ip+"`;"); 
    	doRequestDML(
    		"DROP TABLE IF EXISTS `client_result"+i+"_"+ip+"`;");    	 
   	}			  						
    doRequestDML(
		  "DELETE FROM `client_ip` WHERE `ip` = '"+ip+"';");
	}

	/**
	 * Return the association between TGI & UniGene codes for the specie.
	 * @param specieCode the selected specie
	 * @return a StringBuffer containing code associations (return null if problem)
	 */
	public StringBuffer checkAssociationInfo(String specieCode) {	
		StringBuffer ret = new StringBuffer(
    	"Association for "+specieCode+" : NCBI Unigene / TIGR TGI \n");
    int code_tigr = -1, code_ncbi = -1, last = 0;    
    String pref = "TC";
	 	if (specieCode.equals("Hs")) pref = "THC";
		
		doRequestSEL(
			"SELECT `unigene`, `code_clu_list`"+
			" FROM `"+specieCode+"_clu_nb` "+
			" WHERE  `unigene` > '0' ORDER BY `unigene` ASC;");	
			
    try {    
	    boolean run = results.next(); 
	    ResultSetMetaData resultScheme = results.getMetaData();
	    int nbCol = resultScheme.getColumnCount();
	
	    while(run) {
	    	code_ncbi = results.getInt(1);
	    	code_tigr = results.getInt(2);
				
				if (code_ncbi == last) {
					ret.append("\t" + pref + code_tigr);
					last = code_ncbi;	
				}
	      else {
	      	ret.append(
	      		"\n" + specieCode + "." + code_ncbi + "\t" + pref + code_tigr); 
	      	last = code_ncbi;	
	      }
	      run = results.next();
	    }
	    return ret;
	  }
    catch (SQLException e) { 
			PrintSQLError(e); 
			return null;
	  }
	}
	
	/**
	 * Get the parsed result of a selection request in stringbuffer.
	 * @param results the result of the request
	 * @return the result of the request in a stringbuffer
	 */
	private StringBuffer getResultsInStringBuffer(ResultSet results) {
    StringBuffer ret = new StringBuffer("");
    try {    
	    boolean run = results.next(); 
	    ResultSetMetaData resultScheme = results.getMetaData();
	    int nbCol = resultScheme.getColumnCount();
	
	    while(run) { 
	      for(int i = 1; i <= nbCol; ++i) { 
	      	if (i == nbCol) ret.append(results.getObject(i) + "\n");
	        else ret.append(results.getObject(i) + "\t");   		
	      }
	      run = results.next();
	    }
	    return ret;
	  }
    catch (SQLException e) { 
			PrintSQLError(e); 
			return null;
	  }
	}
	
	/**
	 * Get the parsed result of a selection request in arraylist.
	 * @param results the result of the request
	 * @return the result of the request in an arraylist
	 */
	private ArrayList getResultsInArrayList(ResultSet results) {
  	ArrayList ret = new ArrayList(); 	
  	try {	  	  
	    boolean run = results.next();	  
	    ResultSetMetaData resultScheme = results.getMetaData();
	    int nbCol = resultScheme.getColumnCount();
	    	       
	    while (run) { 
	    	String[] tmpTab = new String[nbCol];
	      for(int i = 1; i <= nbCol; ++i)
	        tmpTab[i-1] = results.getString(i);            						      
	      ret.add(tmpTab);        
	      run = results.next();
	    }   
	    return ret;
	  }
	  catch (SQLException e) { 
			PrintSQLError(e); 
			return null;
	  }
	}
	
	/**
	 * Get the parsed result of a selection request in 2D string table.
	 * @param al an arraylist containing a request result
	 * @return the result of the request in a 2D string table.
	 */
	private String[][] toStringTable(ArrayList al) { 	 		
	  int nbLine = al.size();
	  if (nbLine == 0)
	  	return null;     
	  int nbCol = ((String[])al.get(0)).length;
	  String[][] ret = new String[nbLine][nbCol];	
	    
	  for (int i = 0; i < nbLine; ++i)
	   	ret[i] = (String[])al.get(i);	  
	  return ret;
	}

	/**
	 * Get the content of a HashMap in a 2D object table.
	 * @param hm the hashmap
	 * @return an object[][] construct from the map data
	 * WARNING : 
	 *	key, key, key, key, ...
	 *  val, val, val, val, ...
	 */
	public Object[][] toObjectTable(HashMap hm) { 	 		
	  if (hm == null)
	  	return null;
	  	
	  Object[][] ret = new String[2][hm.size()];	
	  int col = 0;     
		
		for (Iterator i = hm.keySet().iterator(); i.hasNext();) {
			ret[0][col] = (String)i.next();
			col++;
		}
		col = 0; 
		for (Iterator j= hm.values().iterator();j.hasNext();) {
			ret[1][col] = (String)j.next();
			col++;
		}  
	  return ret;
	}
	
	/**
	 * Save the result of a selection request in a HashMap and return it.
	 * @param results the ResultSet object containing the result of request
	 * @param position gives the (key,value) position
	 * @return a HashMap (key, value) containing the result of the request
	 */
	private HashMap getResultsInHashMap(ResultSet results, boolean position) throws SQLException {
  	HashMap ret = new HashMap();  	
    boolean run = results.next();            
	  
	  if (!position) {   
	    while (run) {
	    	String val = results.getString(1);
	    	String key = results.getString(2);
	    	ret.put(key, val);
	      run = results.next();
	    }
	  }
	  else {  
	    while (run) {
	    	String key = results.getString(1);
	    	String val = results.getString(2);	    	
	    	ret.put(key, val);
	      run = results.next();
	    }
	  }
    return ret;
	}
}

