package client;

import java.sql.*;
import java.util.HashMap;
import java.util.ArrayList;

/**
 * The ClientRequestSearch's class is used to execute 
 *	the search process requests on the database.
 */
public class ClientRequestSearch extends ClientRequest {
	
	/**
	 * Construct a new ClientRequestSearch object.
	 * @param con the established connection 
   * @param cp the current parameters for the program
   */
	public ClientRequestSearch(Connection con, ClientParameters cp) {
		super(con, cp);   	
	}
	
	/**
	 * Compute the library list which are associated 
	 *	with selected tissue or development stage encapsulated in the parameter.
	 *	Then, construct a temporary table with these library codes.
	 * @param param a Search parameter object
	 * @param specieCode the specie code to search
	 * @param clientIP the ip of the connected user
	 */
	public void searchStep1(ClientSearchParameter param, 
																			String specieCode, String clientIP) {				
		if (!param.getType())
			doRequestSEL(
				"SELECT `code_lib` FROM `"+specieCode+"_lib` "+
				" WHERE (`tissue` = '"+param.getHValue()+"');");		    
		else 
		  doRequestSEL(
				"SELECT `code_lib` FROM `"+specieCode+"_lib` "+
				" WHERE (`devstage` = '"+param.getHValue()+"');");
	  createSearchStep1TemporaryTable(clientIP);	
	}
	
	/**
	 * Create the temporary table as result of step 1 in a search processing.
	 * @param ip the client ip to use as suffix of table name
	 */
	private void createSearchStep1TemporaryTable(String ip) {
    try {   
			doRequestDML(
				"DROP TABLE IF EXISTS `client_tmp_"+ip+"`;");
			
			doRequestDML( 
				"CREATE TABLE `client_tmp_"+ip+"` ( "+
				" `tmp_code_lib` MEDIUMINT NOT NULL, "+
				" PRIMARY KEY (`tmp_code_lib`));");	
			lockTable("client_tmp_"+ip);
							
			boolean run = results.next(); 
	    while(run) { 
	      int num = results.getInt(1);	      
      	doRequestDML( 
					"INSERT INTO `client_tmp_"+ip+"` (`tmp_code_lib`) "+
					" VALUES ('"+num+"');");
	      run = results.next();
	    }
	    unlockTables();
	  }
    catch (SQLException e) { 
			PrintSQLError(e); 
	  }
	}
	
	/**
	 * Compute the cluster list and the number of EST lists associated 
	 *	to these clusters which match with the selected tissue or devStage 
	 *	found at search step 1.
	 * This request counts the number of EST matching for the tissue or devStage.
	 * Then construct a 'backtracking' temporary table to maintain results (don't
	 *	matter for the condition "=, <, >" and the % value wanted.
	 * 
	 * The integer numSearch is used to know if it is the first search or not, 
	 *	in this last case we work on the subset already created at previous search.
	 * @param param the search parameter
	 * @param numSearch gives the number of search processing
	 * @param specieCode the specie code to search
	 * @param clientIP the IP address of the client user
	 */
	public void searchStep2(ClientSearchParameter param, int numSearch, 
																			String specieCode, String clientIP) {				
		int previous = numSearch - 1;					
		if (numSearch == 1) {
			String[][] toLock = { 
				{ "client_tmp_"+clientIP , "WRITE" },
				{ specieCode+"_est", "READ" },
				{ specieCode+"_clu_nb", "READ" }
			};
			lockTables(toLock);
			
			doRequestSEL( 
				"SELECT `code_clu`, COUNT(*), `"+specieCode+"_clu_nb`.`nb_est` "+
				" FROM `client_tmp_"+clientIP+"`, "+
				" `"+specieCode+"_est`, `"+specieCode+"_clu_nb` "+
				" WHERE (`code_lib_in` = `tmp_code_lib` "+
				"  AND `code_clu` = `code_clu_list`) "+
				" GROUP BY `code_clu`;");
		}	
		else {
			String[][] toLock = { 
				{ "client_tmp_"+clientIP , "WRITE" },
				{ "client_result"+previous+"_"+clientIP , "READ" },
				{ specieCode+"_est", "READ" },
				{ specieCode+"_clu_nb", "READ" }
			};
			lockTables(toLock);
			
			doRequestSEL( 
				"SELECT `cluster`, COUNT(*), `"+specieCode+"_clu_nb`.`nb_est` "+
				" FROM `client_tmp_"+clientIP+"`, `"+specieCode+"_est`, "+
				" `"+specieCode+"_clu_nb`, `client_result"+previous+"_"+clientIP+"` "+
				" WHERE (`code_lib_in` = `tmp_code_lib` "+
				"  AND `code_clu` = `cluster` AND `code_clu` = `code_clu_list`) "+
				" GROUP BY `cluster`;");	 
		} 
		unlockTables();					
		createSearchStep2TemporaryTable(clientIP, numSearch);			
	}
	
	/**
	 * Create the 'backtracking' temporary table as result 
	 *	of step 2 in the search processing.
	 *	NOTE: The 'backtracking' ncresult table is created (don't
	 *	matter for the condition "=, <, >" and the % value wanted.
	 * @param ip the client ip to use as suffix of table name
	 * @param numResult the number in sub-search proccess
	 */
	private void createSearchStep2TemporaryTable(String ip, int numResult) {
    try {   
			int clu = -1, nbEst = -1, totEst = -1, totGo = -1, unigene = -1;
			
			doRequestDML(
				"DROP TABLE IF EXISTS `client_ncresult"+numResult+"_"+ip+"`;");
			doRequestDML(
				"CREATE TABLE `client_ncresult"+numResult+"_"+ip+"` ( "+
				" `tmp_code_clu` MEDIUMINT UNSIGNED NOT NULL, "+
				" `tmp_nb_est_match` SMALLINT UNSIGNED NOT NULL, "+
				" `tmp_nb_est_total` SMALLINT UNSIGNED NOT NULL, "+
				" PRIMARY KEY (`tmp_code_clu`));");
			lockTable("client_ncresult"+numResult+"_"+ip);
			
			boolean run = results.next(); 
			while(run) { 
	      clu = results.getInt(1);
				nbEst = results.getInt(2);
				totEst = results.getInt(3);
				
				doRequestDML(
					"INSERT INTO `client_ncresult"+numResult+"_"+ip+"` "+
					" (`tmp_code_clu`, `tmp_nb_est_match`, `tmp_nb_est_total`) "+
					" VALUES ('"+clu+"', '"+nbEst+"', '"+totEst+"');");	      
	      run = results.next();	
	    }
	    unlockTables();
	  }
    catch (SQLException e) { 
			PrintSQLError(e); 
	  }
	} 
	
	/**
	 * Final Step of the search process : collect cluster informations and
	 *	compute % of appartenance. Select records which respect the condition.
	 * This step is done by filtering the last temporary table
	 *	with the operation selected (=, > or <) and the value enterred (in %)
	 * Here, a new table is created to maintain result and to allow users
	 *	to modify the operation or/and percent value of this search without
	 *	computing all from the beginning. 
	 * @param param a Search parameter object
	 * @param numResult define the number of search processing
	 * @param clientIP the ip of the connected user
	 */
	public void searchStep3(ClientSearchParameter param, int numResult, 
																			String specieCode, String clientIP) {
		if (numResult	== 1) {
			String[][] toLock = { 
				{ "client_ncresult"+numResult+"_"+clientIP , "READ" },
				{ specieCode+"_clu_nb", "READ" }
			};
			lockTables(toLock);

			doRequestSEL( 
				"SELECT `tmp_code_clu`, `tmp_nb_est_match`, `tmp_nb_est_total`, "+
				" `nb_go`, `unigene`"+
				" FROM `client_ncresult"+numResult+"_"+clientIP+"`, `"+specieCode+"_clu_nb`  "+
				" WHERE ((`code_clu_list` = `tmp_code_clu`) AND "+
				" ((`tmp_nb_est_match` / `tmp_nb_est_total`) * 100) "+
				" "+param.getOperation()+" "+param.getValue()+");");	
		}	
		else {
			int previous = numResult - 1;
			String[][] toLock = { 
				{ "client_ncresult"+numResult+"_"+clientIP , "WRITE" },
				{ "client_result"+previous+"_"+clientIP, "WRITE" },
				{ specieCode+"_clu_nb", "READ" }
			};
			lockTables(toLock);
				
			doRequestSEL( 
				"SELECT `tmp_code_clu`, `tmp_nb_est_match`, `tmp_nb_est_total`, "+
				" `"+specieCode+"_clu_nb`.`nb_go`, `"+specieCode+"_clu_nb`.`unigene`"+
				" FROM `client_ncresult"+numResult+"_"+clientIP+"`, "+
				" `client_result"+previous+"_"+clientIP+"`, `"+specieCode+"_clu_nb`"+
				" WHERE ((`tmp_code_clu` = `cluster`) AND (`code_clu_list` = `cluster`) AND"+
				" (((`tmp_nb_est_match` / `tmp_nb_est_total`) * 100) "+
				" "+param.getOperation()+" "+param.getValue()+"));");					
		}	
		unlockTables();		
		createSearchResultTable(clientIP, numResult);	
	}
	 
	/**
	 * Create the search result table with original data from 'client_ncresult'.
	 * @param ip the client ip to use as suffix of table name
	 * @param numResult the number in sub-search proccess
	 */ 
	public void createSearchResultTable(String ip, int numResult) {
    try {   
			int clu = -1, nbEst = -1, totEst = -1, totGo = -1, unigene = -1;
				
			doRequestDML(
				"DROP TABLE IF EXISTS `client_result"+numResult+"_"+ip+"`;");
			doRequestDML( 
				"CREATE TABLE `client_result"+numResult+"_"+ip+"` ( "+
				"	`cluster` MEDIUMINT UNSIGNED NOT NULL, "+
				" `nb_est_match` SMALLINT UNSIGNED NOT NULL, "+
				" `nb_est_total` SMALLINT UNSIGNED NOT NULL, "+
				" `nb_go_total` SMALLINT UNSIGNED NOT NULL, "+
				" `unigene` MEDIUMINT UNSIGNED NOT NULL, "+
				" PRIMARY KEY (`cluster`));");
			lockTable("client_result"+numResult+"_"+ip);

			boolean run = results.next(); 
	    while(run) { 
	      clu = results.getInt(1);
				nbEst = results.getInt(2);
				totEst = results.getInt(3);
				totGo = results.getInt(4);
				unigene = results.getInt(5);
							
	      doRequestDML(
					"INSERT INTO `client_result"+numResult+"_"+ip+"` "+
					" (`cluster`, `nb_est_match`, `nb_est_total`, `nb_go_total`, `unigene`) "+
					" VALUES ('"+clu+"', '"+nbEst+"', '"+totEst+"', "+
					" '"+totGo+"', '"+unigene+"');");	      
	      run = results.next();	
	    }
	    unlockTables();
	  }
    catch (SQLException e) { 
			PrintSQLError(e); 
	  }
	}

	/**
	 * Get the cluster set resulting of search proccessing.
	 * @param ip the client ip
	 * @param numResult the number in sub-search proccess
	 * @param csps the search parameter set
	 * @return a ClientClusterSet object containing the search result
	 */
	public ClientClusterSet getSearchResultClusterSet(String ip, int numResult, 
																							ClientSearchParameterSet csps) {
		try {
			int clu = -1, totalEst = -1, totalGo = -1, unigene = -1;
			ClientClusterSet ret = new ClientClusterSet(csps); 
			
			doRequestSEL(
				"SELECT `cluster`, `nb_est_total`, `nb_go_total`, `unigene` "+
				" FROM `client_result"+numResult+"_"+ip+"`;");				

			boolean run = results.next(); 
			while(run) { 
		    clu = results.getInt(1);
				totalEst = results.getInt(2);
				totalGo = results.getInt(3);
				unigene = results.getInt(4);

				ret.addCluster(new ClientCluster(clu, unigene, totalEst, totalGo, ""));									      
		    run = results.next();	
		  }
		  
		  if (ret.getNbCluster() > 0)
		  	ret.setVectorMap(getESTDetails(csps, ip));
		  
		  results.close();
		  return ret;	
    }
    catch (SQLException e) { 
			PrintSQLError(e); 
			return null;		
	 	}
	}
	
	/**
	 * Get the cluster set resulting of search proccessing.
	 * @param ip the client ip
	 * @param numResult the number in sub-search proccess
	 * @param csps the search parameter set
	 * @return a ClientClusterSet object containing the search result
	 */
	public void createUpdateCodeTable(String codeList, 
																							String specieCode, String ip) {
    try {   
			int old_code = -1, new_code = -1;

			String[][] toLock = { 
				{ specieCode+"_old", "READ" }
			};
			lockTables(toLock);
			doRequestSEL(
				"SELECT `old_code_clu`, `new_code_clu`"+
				" FROM `"+specieCode+"_old` "+
				" WHERE "+codeList+";");		
			unlockTables();
				
			doRequestDML(
				"DROP TABLE IF EXISTS `client_tmp_"+ip+"`;");
			doRequestDML( 
				"CREATE TABLE `client_tmp_"+ip+"` ( "+
				"	`old` MEDIUMINT UNSIGNED NOT NULL, "+
				" `new` SMALLINT UNSIGNED NOT NULL, "+
				" PRIMARY KEY (`old`));");
			lockTable("client_tmp_"+ip);

			boolean run = results.next(); 
	    while(run) { 
	      old_code = results.getInt(1);
				new_code = results.getInt(2);
							
	      doRequestDML(
					"INSERT INTO `client_tmp_"+ip+"` "+
					" (`old`, `new`) "+
					" VALUES ('"+old_code+"', '"+new_code+"');");	      
	      run = results.next();	
	    }
	    unlockTables();
	  }
    catch (SQLException e) { 
			PrintSQLError(e); 
	  }
	}
	
	/**
	 * Get the cluster set resulting of update cluster codes proccessing 
	 *	This is the consequence of a data update.
	 * @param ip the client ip
	 * @param numResult the number in sub-search proccess
	 * @param csps the search parameter set
	 * @return a ClientClusterSet object containing the update result
	 */ 
	public ClientClusterSet getUpdateResultClusterSet(String ip, int numResult, 
																							ClientSearchParameterSet csps) {
    try {
			int clu = -1, totalEst = -1, totalGo = -1, unigene = -1, old = -1;
			String desc = "";
			ClientClusterSet ret = new ClientClusterSet(csps); 
			
			doRequestSEL(
				"SELECT `cluster`, `nb_est_total`, `nb_go_total`, `unigene`, `old` "+
				" FROM `client_result"+numResult+"_"+ip+"`, `client_tmp_"+ip+"` "+
				" WHERE `cluster` = `new`;");				

			boolean run = results.next(); 
			while(run) { 
		    clu = results.getInt(1);
				totalEst = results.getInt(2);
				totalGo = results.getInt(3);
				unigene = results.getInt(4);
				old = results.getInt(5);
				
				if (csps.getSpecieSearch().equals("Hs"))
					desc = "(OLD: THC"+old+")";
				else desc = "(OLD: TC"+old+")";
				ret.addCluster(
					new ClientCluster(clu, unigene, totalEst, totalGo, desc));									      
		    run = results.next();	
		  }
		  
		  if (ret.getNbCluster() > 0)
		  	ret.setVectorMap(getESTDetails(csps, ip));
		  
		  results.close();
		  return ret;	
    }
    catch (SQLException e) { 
			PrintSQLError(e); 
			return null;		
	 	}
	}
	
	/**
	 * Get the EST detail of matching (by cluster) for a search.
	 * @param csps the search parameter set.
	 * @param ip the client ip
	 * @return an HashMap containing matching details.
	 */		
	private HashMap getESTDetails(ClientSearchParameterSet csps, String ip) {
		try {			
			// Case 1 : a single parameter search
			if (csps.getNbParameter() == 1) {
				return getDetailsFor1Param(ip);
			}
			// Case 2 : a multi parameter search
			else {
				ClientSearchParameter cspTmp;
				boolean sameType = true;
				boolean tmpType = csps.getSearchParameter(0).getType();
				for (int i = 1; i < csps.getNbParameter(); ++i) {
					cspTmp = csps.getSearchParameter(i);
					if (!(cspTmp.getType() == tmpType)) {
						sameType = false;
						break;
					}
				}
				
				// all the criteria are not from the same type :
				// Tissue types and development stage types are mixed
				if (!sameType) {										
					return getDetailsForDifferentTypes(ip, csps);			
				}
				// All the criteria are from the same type
				// a simple addition is needed
				else {
					return getDetailsForSameTypes(ip, csps.getNbParameter());			  
				}
			}	
		}
		catch (SQLException e) { 
			PrintSQLError(e); 
			return null;		
	 	}	
	 	catch (Exception ex) { 
			return null;		
	 	}	
	}
	
	/**
	 * Get the EST details of matching (by cluster) for a single parameter search.
	 * @param ip the client ip
	 * @return an HashMap containing matching details.
	 */		
	private HashMap getDetailsFor1Param(String ip) throws Exception {
		int clu = 0;
		HashMap ret = new HashMap();
		int[] sumVector;
		
		doRequestSEL(
			"SELECT `cluster`, `nb_est_match` "+
			" FROM `client_result1_"+ip+"`;");	
		
		boolean run = results.next(); 
		while(run) {				
	  	clu = results.getInt(1);
	  	sumVector = new int[2]; 
			sumVector[1] = results.getInt(2);
			sumVector[0] = sumVector[1];
			ret.put(""+clu, sumVector);			
			run = results.next();	
		}
		return ret;	
	}
	
	/**
	 * Get the EST details of matching (by cluster) for a multi-parameters search
	 *	with all parameters are from the same type.
	 * @param ip the client ip
	 * @param nbParams the number of parameters
	 * @return an HashMap containing matching details.
	 */		
	private HashMap getDetailsForSameTypes(String ip, int nbParams) throws Exception {
		int clu = 0;
		HashMap ret = new HashMap();
		int[] sumVector = new int[nbParams + 1];
		
		String select = "", tables = "";
		String where = "(";
		int next = 1;	
		
		for (int i = 1; i <= nbParams; ++i) {
			next = i+1;
			if (next <= nbParams)
				where += " `client_result"+i+"_"+ip+"`.`cluster` = "+
								 " `client_result"+next+"_"+ip+"`.`cluster` AND";
			select += " `client_result"+i+"_"+ip+"`.`nb_est_match`,";
			tables += " `client_result"+i+"_"+ip+"`,";					
		}
		select = select.substring(0, select.length()-1);
		tables = tables.substring(0, tables.length()-1);
		where  = where.substring(0, where.length()-3) + ")";

		doRequestSEL(
			"SELECT DISTINCT `client_result"+nbParams+"_"+ip+"`.`cluster`, "+select+" "+
			" FROM "+tables+" "+
			" WHERE "+where+";");		
				
		boolean run = results.next(); 
		while(run) {
			int total = 0; 
	  	clu = results.getInt(1);			  	
	  	sumVector = new int[nbParams+1];
	  	for (int j = 0; j < nbParams; ++j) {
				sumVector[j+1] = results.getInt(j+2);
				total += sumVector[j+1];
			}
			sumVector[0] = total;
			ret.put(""+clu, sumVector);								      
	  	run = results.next();	
	  }				  
		return ret;			
	}
	
	/**
	 * Get the EST details of matching (by cluster) for a multi-parameters search
	 *	with all parameters are from different types.
	 * @param ip the client ip
   * @param csps the Search Parameter Set
	 * @return an HashMap containing matching details.
	 */		
	private HashMap getDetailsForDifferentTypes(String ip,
															ClientSearchParameterSet csps) throws Exception {
		int clu = 0;
		int nbParams = csps.getNbParameter();
		HashMap ret = new HashMap();
		HashMap mapTotal = new HashMap();
		int[] sumVector = new int[nbParams + 1];
		
		String select = "", tables = "";
		String where = "(`code_lib_in` = `code_lib` AND (";
		String specie = csps.getSpecieSearch();
		
		int next = 1;	
		ClientSearchParameter cspTmp;
		
		//-> changer l'ordre...
		for (int i = 1; i <= nbParams; ++i) {
			cspTmp = csps.getSearchParameter(i-1);
			if (!cspTmp.getType())
				where += " `tissue` = '"+cspTmp.getHValue()+"' OR ";
			else where += " `devstage` = '"+cspTmp.getHValue()+"' OR ";									 
		}
		where = where.substring(0, where.length()-3) + "))";

		doRequestSEL(
			"SELECT `code_clu`, COUNT(*) "+
			" FROM `"+specie+"_est`, `"+specie+"_lib` "+
			" WHERE "+where+" GROUP BY `code_clu`;");		
			 
		int nb = -1;
		clu = -1;	
		boolean run = results.next(); 
		while(run) { 
	    clu = results.getInt(1);
			nb = results.getInt(2);
			mapTotal.put(""+clu, ""+nb);			      
	    run = results.next();	
	  }
	  
		// computing detailed EST results
		where = "(";
		for (int i = 1; i <= nbParams; ++i) {
			next = i+1;
			if (next <= nbParams)
				where += " `client_result"+i+"_"+ip+"`.`cluster` = "+
								 " `client_result"+next+"_"+ip+"`.`cluster` AND";
			select += " `client_result"+i+"_"+ip+"`.`nb_est_match`,";
			tables += " `client_result"+i+"_"+ip+"`,";					
		}
		select = select.substring(0, select.length()-1);
		tables = tables.substring(0, tables.length()-1);
		where  = where.substring(0, where.length()-3) + ")";

		doRequestSEL(
			"SELECT DISTINCT `client_result"+nbParams+"_"+ip+"`.`cluster`, "+select+" "+
			" FROM "+tables+" "+
			" WHERE "+where+";");		
					
		run = results.next(); 
		while(run) { 
	  	clu = results.getInt(1);			  	
	  	sumVector = new int[nbParams+1];
	  	for (int j = 0; j < nbParams; ++j)
				sumVector[j+1] = results.getInt(j+2);
			sumVector[0] = Integer.parseInt((String)mapTotal.get(""+clu));
			ret.put(""+clu, sumVector);								      
	  	run = results.next();	
		}
		return ret;
	}
}

