package client;

import java.io.*;
import java.util.ArrayList;

/**
 * A ClientSearchParameterSet is a set of SearchParameter.
 */
public class ClientSearchParameterSet implements Serializable {		
	private String specie;
	private int ins_NCBI;
	private int ins_TIGR;	
	private ArrayList list;
	
	/**
	 * Construct a new ClientSearchParameterSet object.
	 */ 
	ClientSearchParameterSet() {
		this.specie = "";
		this.list = new ArrayList();
	}
	
	/**
	 * Construct a new ClientSearchParameterSet object.
	 * @param specie the specie to search
	 */ 
	ClientSearchParameterSet(String specie) {
		this.specie = specie;
		this.list = new ArrayList();
	}
	
	/**
	 * Construct a new ClientSearchParameterSet object.
	 * @param specie the specie to search
	 * @param list the list of parameters
	 */
	ClientSearchParameterSet(String specie, ArrayList list) {
		this.specie = specie;
		this.list = list;
	}
	
	/**
	 * Get the name of the specie which is targetted by this search
	 * @return the specie name
	 */
	public String getSpecieSearch() {
		return this.specie;
	}
	
	/**
	 * Get the number of parameters in the list.
	 * @return the number of parameters
	 */
	public int getNbParameter(){
	  return list.size();
	}
	
	/**
	 * Get the NCBI data version.
	 * @return the NCBI version
	 */
	public int getNCBIVersion() {
		return this.ins_NCBI;
	}
	
	/**
	 * Set a new NCBI data version.
	 * @param ins_NCBI the version number
	 */
	public void setNCBIVersion(int ins_NCBI){
	  this.ins_NCBI = ins_NCBI;
	}
	
	/**
	 * Get the TIGR data version.
	 * @return the TIGR version
	 */
	public int getTIGRVersion() {
		return this.ins_TIGR;
	}
	
	/**
	 * Set a new TIGR data version.
	 * @param ins_TIGR the version number
	 */
	public void setTIGRVersion(int ins_TIGR){
	  this.ins_TIGR = ins_TIGR;
	}
	
	/**
	 * Get the parameter's list.
	 * @return an ArrayList containing all the parameters
	 */
	public ArrayList getList() {
	  return this.list;
	}
	
	/**
	 * Set a new parameter's list.
	 * @param list the list of parameters to set
	 */
	public void setList(ArrayList list){
	  this.list = list;
	}

	/**
	 * Add a new search parameter in the list. The adding operation is done 
	 *	to sort parameters according to increasingly wages.
	 * @param sp the search parameter to add in the list
	 */
	public void addNewSearchParameter(ClientSearchParameter sp) {
		int i;
		for (i = 0; i < list.size(); ++i) {
			ClientSearchParameter tmp = (ClientSearchParameter)list.get(i);
			if (sp.getWage() <= tmp.getWage())
				break;		
		}
		list.add(i, sp);
	}
	
	/**
	 * Remove a parameter in the list at a specified position.
	 * @param n the position of the parameter to delete
	 */ 
	public void delSearchParameter(int n) {
		if (!(n < 0 || n >= list.size()))	
			list.remove(n);
	}

	/**
	 * Get the search parameter at a specified position in the list.
	 * @param n the position of the parameter to return
	 * @return a ClientSearchParameter object which is the parameter
	 */
  public ClientSearchParameter getSearchParameter(int n) {
  	if (n >= 0 && n < this.list.size())
  		return ((ClientSearchParameter)list.get(n));
  	else return null;
  }
}