package client;

import java.sql.*;
import java.io.*;
import java.util.zip.*;
import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * The ClientThreadSearch class is used to manage the search process.
 */
public class ClientThreadSearch extends Thread {
	private ClientParameters cp;
	private ClientRequestSearch crm;
	private ClientSearchParameterSet csp;
	private ClientClusterSet ccs;
	private int[] searchVector;
	private String specieCode;
	private String clientIP;
	private long searchTime = -1;
	private String oldCodeList = null;
	
	/**
	 * Construct a new ClientThreadSearch object designed 
	 *	to execute search on the data base.
	 * @param cp the current client parameters
	 * @param csp the set of selected search parameters (sorted by wages)
	 * @param crm the requests to execute for the search process
	 * @param searchVector the backtracking vector
	 * @clientIP the clietn IP address
	 */	
	public ClientThreadSearch(ClientParameters cp, ClientSearchParameterSet csp, 
								ClientRequestSearch crm, int[] searchVector, String clientIP) {
		this.cp = cp;
		this.csp = csp;
		this.crm = crm;
		this.searchVector = searchVector;
		this.specieCode = csp.getSpecieSearch();
		this.clientIP = clientIP;
		ccs = new ClientClusterSet();
	}	
	
	/**
	 * The run() method used to run the search process.
	 */
	public void run() {		
		try {
			ClientTimer.resetTime();
			int i = 0;
			ClientSearchParameter param;	
			
			for (i = 0; i < csp.getNbParameter(); ++i) {
				param = csp.getSearchParameter(i);								
				switch(searchVector[i]) {
					case 0 : // use 'backtracking' ncresult table
						crm.searchStep3(param, i+1, specieCode, clientIP);
						break;
					case 1 : // default search mode					
						crm.searchStep1(param, specieCode, clientIP);											
						crm.searchStep2(param, i+1, specieCode, clientIP);								
						crm.searchStep3(param, i+1, specieCode, clientIP);
						break;
				}
			}
			//update clusters code
			if (oldCodeList != null) {
				crm.createUpdateCodeTable(oldCodeList, specieCode, clientIP);
				this.ccs = crm.getUpdateResultClusterSet(clientIP, i, csp);	
				oldCodeList = null;
			}
			else this.ccs = crm.getSearchResultClusterSet(clientIP, i, csp);	

			searchTime = ClientTimer.stopTime();	
		}
		catch (Exception e) { 
			PrintException(e);
		}
	}
	
	/**
	 * Get the ClusterSet retrieved by the search.
	 * @return a ClientClusterSet object which contains all the clusters matching
	 */
	public ClientClusterSet getClusterSet() {
		return this.ccs;
	}
	
	/**
	 * Set the list of old cluster codes.
	 * @param oldCodeList the list of old codes
	 */
	public void setOldClusterCodes(String oldCodeList) {
		this.oldCodeList = oldCodeList;
	}
	
	
	/**
	 * Get the approximative time for the previous search in seconds.
	 * @return a long giving the search time in seconds.
	 */
	public long getSearchTime() {
		return this.searchTime;
	}
	
	/**
	 * Print a time duration information for an operation.
	 * @param msg the message to print on log file
	 */
	private void PrintTimeInfo(String msg) {
		if (cp.PRINT_DEBUG_INFO) {
		  System.out.println(msg);
	 	}
	}
	
	/**
   * Print the message from an exception if the debug mode is selected.
   * @param ex the exception to print
   */
  private void PrintException(Exception ex) {
		if (cp.PRINT_DEBUG_INFO) {
	  	System.out.println("** CLIENT THREAD SEARCH Exception :");
	    System.out.println("* Message: " + ex.getMessage());
	  }
	}
}