package client;

import java.sql.*;
import javax.sql.*;
import javax.naming.*;
import java.lang.reflect.*;
import java.util.*;
import java.util.Properties;

/**
 * The class ClientUniversalDB allows a connection to a data base
 * 
 * Type supported :
 * MYSQL, for a MySQL 3.x/4.x data base. 
 * ORACLE, for a Oracle 8i/9i data base.
 * SQL_SERVER_2000, for a Microsoft SQL Server 2000.
 * ODBC, is slower but work with all ODBC source.
 */
public class ClientUniversalDB {
	private Hashtable suportedDataBase = new Hashtable();
	private Properties properties = new Properties();

	private String login = "";
	private String password = "";
	private String type = "";
	private String name = "";
	private String server = "";
	private String port = "";
	private String option = "";
	
	/**
	 * Construct a new ClientUniversalDB object.
	 */
	public ClientUniversalDB() {
		this(null, null, null);
	}
	
	/**
	 * Construct a new ClientUniversalDB.
	 * @param name the name of the data base
	 * @param server the name or IP address of the server
	 * @param port the port of the data base 
	 */
	public ClientUniversalDB(String name, String server, String port) {
		this(null, name, server, port);
	}
	
	/**
	 * Construct a new ClientUniversalDB.
	 * @param type the data base type, default = UniversalDB.ODBC.
	 * @param name the name of the data base
	 * @param server the name or IP address of the server
	 * @param p the port of the data base 
	 */
	public ClientUniversalDB(String type, String name, String server, String p) {
		this(null, null, type, name, server, p);
	}
	
	/**
	 * Construct a new ClientUniversalDB.
	 * @param login the login
	 * @param password the password
	 * @param type the data base type, default = UniversalDB.ODBC.
	 * @param name the name of the data base
	 * @param server the name or IP address of the server
	 * @param port the port of the data base 
	 */
	public ClientUniversalDB(String login, String password, String type, 
																		String name, String server, String port) {
		initSuportedDataBase();		
		setLogon(login, password);
		setType(type);
		setName(name);
		setServer(server);
		setPort(port);
	}
	
	/**
	 * Sets the login and password for the connection to the data base.
	 * @param login the login
	 * @param password the password
	 */
	public void setLogon(String login, String password) {
		setLogin(login);
		setPassword(password);
	}
	
	/**
	 * Sets the login for the connection to the data base.
	 * @param login the login
	 */
	public void setLogin(String login) {
		if(login == null) {
			setLogin("");
			return;
		}
		this.login = login;
	}
	
	/**
	 * Get the login for the connection to the data base.
	 * @return the login
	 */
	public String getLogin() {
		return(login);
	}
	
	/**
	 * Set the password for the connection to the data base.
	 * @param password the password
	 */
	public void setPassword(String password) {
		if(password == null) {
			setPassword("");
			return;
		}
		this.password = password;
	}
	
	/**
	 * Get the password for the connection to the data base.
	 * @return the password
	 */
	public String getPassword(){
		return(password);
	}
	
	/**
	 * Set the data base name
	 * @param name the name of the data base
	 */
	public void setName(String name) {
		if(type == null) {
			setName("");
			return;
		}
		this.name = name;
	}
	
	/**
	 * Get the data base name
	 * @return the name of the data base
	 */
	public String getName() {
		return(name);
	}
	
	/**
	 * Set the type of data base :
	 * Type supported :
	 * UniversalDB.MYSQL, for a MySQL 3.x/4.x data base. 
	 * UniversalDB.ORACLE, for a Oracle 8i/9i data base.
	 * UniversalDB.SQL_SERVER_2000, for a Microsoft SQL Server 2000.
	 * UniversalDB.ODBC, is slower but work with all ODBC source.
	 * @param type the data base type, default = UniversalDB.ODBC.
	 */
	public void setType(String type) {
		if(type == null) {
			setType("odbc");
			return;
		}
		this.type = type;
	}
	
	/**
	 * Get the type of data base used.
	 * @return the type of data base used.
	 */
	public String getType() {
		return this.type;
	}
	
	/**
	 * Set the server name or IP address. The default is localhost.
	 * @param server the name or IP address of the server
	 */
	public void setServer(String server) {
		if(server == null) {
			setServer("localhost");
			return;
		}
		this.server = server;
	}
	
	/**
	 * Get the server name or IP address.
	 * @return the name or IP address of the server
	 */
	public String getServer() {
		return this.server;
	}
	
	/**
	 * Set the server port.
	 * @param port the port of the data base
	 */
	public void setPort(String port) {
		if(port == null) {
			setPort("");
			return;
		}
		this.port = port;
	}
	
	/**
	 * Get the server port.
	 * @return the port of the data base
	 */
	public String getPort() {
		return this.port;
	}
	
	/**
	 * Set the option string for the url connection
	 * @param option the options
	 */
	public void setOption(String option) {
		if(option == null) {
			setOption("");
			return;
		}
		this.option = option;
	}
	
	/**
	 * Get the option string for the url connection
	 * @return the option string
	 */
	public String getOption() {
		return this.option;
	}
	
	/**
	 * Initialize the hashtable containing possible drivers.
	 */
	private void initSuportedDataBase() {
		suportedDataBase.put("mysql", "com.mysql.jdbc.Driver");
		suportedDataBase.put("microsoft", "com.microsoft.jdbc.sqlserver.SQLServerDriver");
		suportedDataBase.put("oracle", "oracle.jdbc.driver.OracleDriver");
		suportedDataBase.put("odbc", "sun.jdbc.odbc.JdbcOdbcDriver");
	}
	
	/**
	 * Add a new driver to the suported driver list.
	 * The type is the string used to get connection.
	 * Example :
	 * 	ODBC data base = odbc 
	 * 	MySQL data base = mysql
	 * 	the driverName is the string that represent the name of the driver.
	 * Example :
	 * 	ODBC data base = sun.jdbc.odbc.JdbcOdbcDriver
	 * 	MySQL data base = com.mysql.jdbc.Driver
	 * @param type the type of data base
	 * @param driverName the name of the driver
	 */
	public void addDriver(String type, String driverName) {
		suportedDataBase.put(type, driverName);
	}
	
	/**
	 * Remove a driver from the suported driver list.
	 * @param type the type of data base
	 */
	public void delDriver(String type) {
		suportedDataBase.remove(type);
	}
	
	/**
	 * Get all suported type of data base.
	 * @return a enumeration of the suported type of data base
	 */
	public Enumeration getAllType() {
		return suportedDataBase.keys();
	}
	
	/**
	 * Test if the type is suported.
	 * @return true if the type is suported
	 */
	public boolean isSuportedType(String type) {
		return(suportedDataBase.containsKey(type));
	}
	
	/**
	 * Get the driver string for the specified type.
	 * @param type the type of data base
	 * @return the driver string
	 */
	public String getDriverName(String type) {
		return (String)suportedDataBase.get(type);
	}  
   	
  /**
   * Create a connection.
   * @return the connection
  */
	public Connection connect() throws Exception {
		StringBuffer url = new StringBuffer();
		String port = null;
		Connection connection = null;
		
		Class.forName(getDriverName(getType()));
		
		// create url connection
		url.append("jdbc:");
		url.append(getType() + ":");
		if(!getType().equals("odbc")) {
			if(!getPort().equals("")) {
				port = new String(":" + getPort());
			}
			if(getType().equals("mysql")) {
				url.append("//" + getServer() + port);
			}
			if(getType().equals("microsoft")) {
				url.append("sqlserver://" + getServer() + port);
			}
		}
		if(!getName().equals("")) {
			if(getType().equals("odbc")) {
				url.append(name);
			}
			if(getType().equals("mysql")) {
				url.append("/" + name);
			}
			if(getType().equals("microsoft")) {
				url.append(";DatabaseName=" + name);
			}
		}		
    properties.setProperty("user",login);
    properties.setProperty("password",password);

		//connect to the data base by using the selected driver.
		connection = DriverManager.getConnection(url.toString(), properties);
		return connection;
	}
}